@@ -30,11 +30,10 @@ def main():
     nt = 40000    # number of iterations
 
     # Setup
-    mg = RasterModelGrid()
-    mg.initialize(nr, nc, dx)
-    u = mg.create_cell_dvector()
+    mg = RasterModelGrid(nr, nc, dx)
+    u = mg.zeros(centering='cell')
     interior_cells = mg.get_interior_cells()
-    dudt = mg.create_cell_dvector()
+    dudt = mg.zeros(centering='cell')
 
     # Boundaries
     mg.set_noflux_boundaries(False, True, False, True)
