@@ -36,6 +36,32 @@ def simple_poly_area(x, y):
                     x[-1] * y[0] - x[0] * y[-1])
 
 
+def calculate_link_lengths(pts, link_from, link_to):
+    """
+    Calculates and returns length of links between nodes.
+    
+    Inputs: pts - Nx2 numpy array containing (x,y) values
+            link_from - 1D numpy array containing index numbers of nodes at 
+                        starting point ("from") of links
+            link_to - 1D numpy array containing index numbers of nodes at 
+                      ending point ("to") of links
+                      
+    Returns: 1D numpy array containing horizontal length of each link
+    
+    Example:
+        
+        >>> pts = numpy.array([[0.,0.],[3.,0.],[3.,4.]]) # 3:4:5 triangle
+        >>> lfrom = numpy.array([0,1,2])
+        >>> lto = numpy.array([1,2,0])
+        >>> calculate_link_lengths(pts, lfrom, lto)
+        array([ 3.,  4.,  5.])
+    """
+    dx = pts[link_to, 0] - pts[link_from, 0]
+    dy = pts[link_to, 1] - pts[link_from, 1]
+    link_length = numpy.sqrt(dx * dx + dy * dy)
+    return link_length
+    
+    
 class VoronoiDelaunayGrid(ModelGrid):
     """
     This inherited class implements an unstructured grid in which cells are
@@ -43,10 +69,16 @@ class VoronoiDelaunayGrid(ModelGrid):
     scipy.spatial module to build the triangulation.
     
     """
-    def __init__(self):
-        pass
+    #print 'VoronoiDelaunayGrid.__init__'
+    
+    def __init__(self, x=None, y=None, **kwds):
         
-    def initialize(self, x, y):
+        if (x is not None) and (y is not None):
+            self._initialize(x, y)
+        super(VoronoiDelaunayGrid, self).__init__(**kwds)
+        
+        
+    def _initialize(self, x, y):
         """
         Creates an unstructured grid around the given (x,y) points.
         """
@@ -77,24 +109,25 @@ class VoronoiDelaunayGrid(ModelGrid):
         #   - all cells are active (later we'll build a mechanism for the user
         #       specify a subset of cells as active)
         #
-        self.num_nodes = len(x)
+        self._num_nodes = len(x)
         #print x, y
         self._node_x = x
         self._node_y = y
         [self.node_status, self.interior_nodes, self.boundary_nodes] = \
                 self.find_perimeter_nodes(pts)
-        self.num_cells = len(self.interior_nodes)
-        self.num_activecells = self.num_cells
-        [self.node_cell, self.cell_node] = self.setup_node_cell_connectivity( \
-                                            self.node_status, self.num_cells)
+        self._num_active_nodes = len(self.interior_nodes)
+        self._num_cells = len(self.interior_nodes)
+        self._num_active_cells = self.number_of_cells
+        [self.node_cell, self.cell_node] = self.setup_node_cell_connectivity(
+            self.node_status, self.number_of_cells)
         self.node_activecell = self.node_cell
         self.activecell_node = self.cell_node
-        
+
         # ACTIVE CELLS: Construct Voronoi diagram and calculate surface area of
         # each active cell.
         from scipy.spatial import Voronoi
         vor = Voronoi(pts)
-        self.active_cell_areas = numpy.zeros(self.num_activecells)
+        self.active_cell_areas = numpy.zeros(self.number_of_active_cells)
         for node in self.activecell_node:
             xv = vor.vertices[vor.regions[vor.point_region[node]],0]
             yv = vor.vertices[vor.regions[vor.point_region[node]],1]
@@ -104,21 +137,23 @@ class VoronoiDelaunayGrid(ModelGrid):
         # "from" and "to" nodes.
         [self.link_fromnode, self.link_tonode, self.active_links, self.face_width] \
                 = self.create_links_and_faces_from_voronoi_diagram(vor)
-        self.num_links = len(self.link_fromnode)
+        self._num_links = len(self.link_fromnode)
+        self._num_faces = self._num_links # temporary: to be done right!
                     
         # LINKS: Calculate link lengths
-        self.link_length = self.calculate_link_lengths(pts, self.link_fromnode, 
-                                                       self.link_tonode)
+        self._link_length = calculate_link_lengths(pts, self.link_fromnode,
+                                                  self.link_tonode)
                                                        
         # LINKS: inlink and outlink matrices
-        self.setup_inlink_and_outlink_matrices()
+        self._setup_inlink_and_outlink_matrices()
         
         # ACTIVE LINKS: Create list of active links, as well as "from" and "to"
         # nodes of active links.
-        self.reset_list_of_active_links()
+        self._reset_list_of_active_links()
 
         # LINKS: ID of corresponding face, if any
-        self.link_face = numpy.zeros(self.num_links, dtype=int)+BAD_INDEX_VALUE  # make the list
+        self.link_face = (numpy.zeros(self.number_of_links, dtype=int) +
+                          BAD_INDEX_VALUE)  # make the list
         face_id = 0
         for link in self.active_links:
             self.link_face[link] = face_id
